package cn.com.ty.lift.business.annualinspection.controller;


import cn.com.ty.lift.business.annualinspection.dto.InspectionRequest;
import cn.com.ty.lift.business.annualinspection.dto.InspectionResponse;
import cn.com.ty.lift.business.annualinspection.entity.AnnualInspection;
import cn.com.ty.lift.business.annualinspection.service.impl.AnnualInspectionServiceImpl;
import cn.com.ty.lift.business.common.CountPage;
import cn.com.ty.lift.business.common.RestResult;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

/**
 * <p>
 * 年检记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-10
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("/annualinspection/annual-inspection")
public class AnnualInspectionController {

    private AnnualInspectionServiceImpl annualInspectionService;


    /**
     * @description 根据id查询
     * @date 2019/12/10 10:19
     * @param
     * @return
     */
    @PostMapping("findOne")
    public RestResponse findOne(@RequestBody InspectionRequest request){
        Long id = request.getId();
        if(null == id || id <= 0){
            return RestResult.errParam();
        }
        InspectionResponse entity = annualInspectionService.selectByIdWithInfo(id);
        if(null == entity){
            return RestResult.fail();
        }
        return RestResult.okData(entity);
    }

    /**
     * @description 根据条件查询年检记录
     * @date 2019/12/11 17:15
     * @param
     * @return
     */
    @PostMapping("page")
    public RestResponse page(@RequestBody InspectionRequest request){
        if(null == request){
            return RestResult.errParam();
        }
        String info = request.getInfo();
        if(StrUtil.isNotBlank(info)){
            request.setInfo(StrUtil.format("%{}%",info));
        }

        IPage<InspectionResponse> pages = annualInspectionService.listByCondition(request);
        if(null == pages){
            return RestResult.fail();
        }
        long count = annualInspectionService.countByCondition(request);
        CountPage<InspectionResponse> countPage = new CountPage<>(count, pages);
        return RestResult.okData(countPage);
    }

    /**
     * @description 修改年检信息
     *
     * status 状态（0 待完成；1 已完成；2 超期）
     *
     * stepStatus  :年检阶段状态
     *
     * 第一阶段 1，第二阶段 2-3，第三阶段 4-7，第四阶段 8-10
     *
        1  维保工确认年检计划
        --------------------------------
        2  维保工上传自检报告
        3  企业文员确认开始执行年检时间
        --------------------------------
        4	维保工上传检验结果
        5	企业文员确认检验结果-合格 --> 不能算年检超期
        6	企业文员确认检验结果-整改 --> 回到第二阶段
        7	企业文员确认检验结果-不合格 --> 可以算年检超期
        --------------------------------
        8	维保工上传年检报告和合格证
        9	企业文员确认年检报告和合格证-合格
        10	企业文员确认年检报告和合格证-不合格 -->停用电梯
     * @date 2019/12/10 10:04
     * @param
     * @return
     */
    @PostMapping("modify")
    public RestResponse modify(@RequestBody AnnualInspection entity){
        if(null == entity){
            return RestResult.errParam();
        }

        Long id = entity.getId();
        if(null == id || id <= 0){
            return RestResult.errParam();
        }
        AnnualInspection old = annualInspectionService.getById(id);
        if(null == old){
            return RestResult.errParam();
        }
        Integer oldStepStatus = old.getStepStatus();
        Integer stepStatus = entity.getStepStatus();

        //1 维保工确认年检计划
        if(oldStepStatus == 0 && (stepStatus != 0 && stepStatus != 1)){
            return RestResult.err("请执行[维保工确认年检计划]操作");
        }

        //2  维保工上传自检报告
        if(oldStepStatus == 1 && stepStatus != 2){
            return RestResult.err("请执行[维保工上传自检报告]操作");
        }
        //3  企业文员确认开始执行年检时间
        if(oldStepStatus == 2 && stepStatus != 3){
            return RestResult.err("请执行[企业文员确认开始执行年检时间]操作");
        }
        if (stepStatus == 3 && entity.getBeginInspectionDate() == null){
            return RestResult.err("请设置开始执行年检时间");
        }
        // 4  维保工上传检验结果
        if(oldStepStatus == 3 && stepStatus != 4){
            return RestResult.err("请执行[维保工上传检验结果]操作");
        }
        /**
         *  5	企业文员确认检验结果-合格
         *  6	企业文员确认检验结果-整改
         *  7	企业文员确认检验结果-不合格
         */
        if(oldStepStatus == 4 && (stepStatus !=5 && stepStatus != 6 && stepStatus !=7)){
            return RestResult.err("请执行[企业文员确认检验结果]操作");
        }
        /**
         * 5	企业文员确认检验结果-合格 --> 8	维保工上传年检报告和合格证
         * 7	企业文员确认检验结果-不合格-->8	维保工上传年检报告和合格证
         */
        if((oldStepStatus == 5 || oldStepStatus == 7) && stepStatus != 8){
            return RestResult.err("请执行[维保工上传年检报告和合格证]操作");
        }

        // 6	企业文员确认检验结果-整改 --> 4	维保工上传检验结果
        if(oldStepStatus == 6 && stepStatus != 4){
            return RestResult.err("请执行[维保工上传检验结果]操作");
        }
        /**
         * 8	维保工上传年检报告和合格证--> 9  企业文员确认年检报告和合格证 -合格
         * 8	维保工上传年检报告和合格证--> 10  企业文员确认年检报告和合格证 -不合格
         */
        if(oldStepStatus == 8 && (stepStatus != 9 && stepStatus !=10)){
            return RestResult.err("请执行[企业文员确认年检报告和合格证]操作");
        }
        //合格：文员确认年检报告并设置下次年检时间
        if(stepStatus == 9 && null == entity.getNextInspectionTime()){
            return RestResult.err("请设置下次年检时间");
        }

        if(stepStatus == 0){
            entity.setStepName(0);
        }else if(stepStatus == 1){
            entity.setStepName(1);
        }else if(stepStatus == 2 || stepStatus == 3){
            entity.setStepName(2);
        }else if(stepStatus >= 4 && stepStatus <= 7){
            entity.setStepName(3);
        }else if(stepStatus >= 8 && stepStatus <= 10){
            entity.setStepName(4);
        }else{
            return RestResult.err("年检状态有误:" + stepStatus);
        }
        // 设置年检完成 status = 1
        entity.setFinishTime(LocalDateTime.now());
        entity.setStatus(1);

        boolean result = annualInspectionService.saveOrUpdate(entity);
        if(result){
            //todo 不合格：流程结束，电梯扭转到不合格状态列表，电梯停保，清理维保计划记录
            if(stepStatus == 10){

            }
            return RestResult.okData(result);
        }
        return RestResult.fail();
    }
}
