package cn.com.ty.lift.system.user.service.impl;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.utils.ProjectUtils;
import cn.com.ty.lift.system.constants.CommonConstants;
import cn.com.ty.lift.system.user.dao.entity.*;
import cn.com.ty.lift.system.user.dao.entity.model.*;
import cn.com.ty.lift.system.user.dao.mapper.MtCompanyUserMapper;
import cn.com.ty.lift.system.user.service.*;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author huangyuan
 * @date 2019-12-03
 * @description
 */
@Service
public class MtCompanyUserService extends ServiceImpl<MtCompanyUserMapper, MtCompanyUser> implements IMtCompanyUserService {

    @Autowired
    private IUserRoleService userRoleService;

    @Autowired
    private IUserInfoService userInfoService;

    @Autowired
    private IProjectService projectService;

    @Autowired
    private IRoleService roleService;

    @Autowired
    private IUserService userService;

    @Override
    @Transactional
    public RestResponse assignRole(MtCompanyUserRequest mtCompanyUserRequest) {
        //获取用户是否已有mtCompanyUser信息。
        MtCompanyUser mtCompanyUser = this.getOne(new QueryWrapper<MtCompanyUser>()
                .eq("mt_company_id", mtCompanyUserRequest.getCompanyId())
                .eq("user_id", mtCompanyUserRequest.getUserId())
        );
        if (mtCompanyUser == null) {
            mtCompanyUser = new MtCompanyUser(mtCompanyUserRequest);
            mtCompanyUser.setMtCompanyId(mtCompanyUserRequest.getCompanyId());
        }
        //保存职业信息
        this.saveOrUpdate(mtCompanyUser);
        //保存用户角色关联信息
        //1.删除原有的关联关系
        UserRole queryUserRole = new UserRole(mtCompanyUserRequest.getUserId(), mtCompanyUserRequest.getRoleId());
        UserRole userRole = userRoleService.getByUserRole(queryUserRole);
        if (userRole != null) {
            boolean deleteUserRoleFlag = userRoleService.removeById(userRole.getId());
            if (!deleteUserRoleFlag) {
                return RestResponse.error(ApiConstants.RESULT_ERROR, "删除原有关联关系失败");
            }
        }
        //保存新的关联关系
        userRole = new UserRole();
        userRole.setCompanyId(mtCompanyUserRequest.getCompanyId());
        userRole.setRoleId(mtCompanyUserRequest.getRoleId());
        userRole.setUserId(mtCompanyUserRequest.getUserId());
        userRoleService.save(userRole);
        return RestResponse.ok(null, ApiConstants.RESULT_SUCCESS, "分配角色成功");
    }

    @Override
    public RestResponse getCompanyMembers(MtCompanyUserRequest mtCompanyUserRequest) {
        List<UserResponse> userResponseList = new ArrayList<>();
        List<MtCompanyUser> mtCompanyUserList = this.list(new QueryWrapper<MtCompanyUser>()
                .eq("mt_company_id", mtCompanyUserRequest.getCompanyId()));
        if (mtCompanyUserList != null && mtCompanyUserList.size() > 0) {
            //获取用户id集合
            List<Long> userIdList = ProjectUtils.getAttrList(mtCompanyUserList, "userId", null);
            //获取用户信息集合
            List<UserInfo> userInfoList = (List<UserInfo>) userInfoService.listByIds(userIdList);
            Map<Long, UserInfo> userIdToUserInfo = ProjectUtils.attrToObjMap(userInfoList, "userId", null);
            if (userIdToUserInfo != null && userIdToUserInfo.size() > 0) {
                for (Long userId : userIdList) {
                    //封装用户信息
                    UserResponse userResponse = new UserResponse();
                    userResponse.setUserId(userId);
                    UserInfo userInfo = userIdToUserInfo.get(userId);
                    if (userInfo != null) {
                        userResponse.setName(userInfo.getName());
                    }
                    userResponseList.add(userResponse);
                }
            }
        }
        return RestResponse.ok(userResponseList, ApiConstants.RESULT_SUCCESS, "获取团队用户成功");
    }

    @Override
    public RestResponse kickOutTeam(MtCompanyUserRequest mtCompanyUserRequest) {
        //判断用户是否可以踢出团队
        ProjectUser projectUser = new ProjectUser(mtCompanyUserRequest.getUserId(), mtCompanyUserRequest.getCompanyId());
        Map<String, String> judgeMsg = projectService.judgeKickOutTeam(projectUser);
        //获取判断返回的状态码
        String code = judgeMsg.get(CommonConstants.RETURN_CODE_FIELD);
        if (ApiConstants.RESULT_ERROR.equals(code)) {
            return RestResponse.error(ApiConstants.RESULT_ERROR, judgeMsg.get(CommonConstants.RETURN_MSG_FIELD));
        }
        MtCompanyUser mtCompanyUser = this.getById(mtCompanyUserRequest.getId());
        mtCompanyUser.setStatus(ApiConstants.ApplicationConstants.COMPANY_USER_FAIL);
        this.updateById(mtCompanyUser);
        return RestResponse.ok(null, ApiConstants.RESULT_SUCCESS, "踢出用户成功");
    }

    @Override
    public RestResponse getCompanyUserList(MtCompanyUserRequest mtCompanyUserRequest) {

        List<MtCompanyUserResponse> mtCompanyUserResponseList = new ArrayList<>();
        IPage mtCompanyUserPage = new Page(mtCompanyUserRequest.getPageNum(), mtCompanyUserRequest.getPageSize());
        mtCompanyUserPage = this.page(mtCompanyUserPage, new QueryWrapper<MtCompanyUser>()
                .eq("mt_company_id", mtCompanyUserRequest.getCompanyId())
                .eq("delete_flag", ApiConstants.DELETE_NO)
        );
        List<MtCompanyUser> mtCompanyUserList = mtCompanyUserPage.getRecords();
        if (mtCompanyUserList != null && mtCompanyUserList.size() > 0) {

            /** 用户信息区域 **/
            //获取用户id集合
            List<Long> userIdList = ProjectUtils.getAttrList(mtCompanyUserList, "userId", null);
            Map<Long, UserResponse> userIdToUserResponse = userService.getUserResponseByUserIdList(userIdList);

            /** 角色信息区域 **/
            //获取用户角色关联关系
            Map<Long, Role> userIdToRole = roleService.getUserIdToRoleByUserIdsAndCompanyId(userIdList, mtCompanyUserRequest.getCompanyId());

            /** 操作证信息区域 **/
            //获取用户操作证信息
            List<LiftCertificate> liftCertificateList = projectService.getLiftCertificateList(new ProjectUser(mtCompanyUserRequest.getCompanyId(), userIdList));
            Map<Long, LiftCertificate> userIdToLiftCertificate = ProjectUtils.attrToObjMap(liftCertificateList, "userId", null);

            /** 公司用户信息组装区域 **/
            //组装公司用户信息
            for (MtCompanyUser mtCompanyUser : mtCompanyUserList) {
                MtCompanyUserResponse mtCompanyUserResponse = new MtCompanyUserResponse();
                UserResponse userResponse = userIdToUserResponse.get(mtCompanyUser.getUserId());
                //设置用户角色信息
                mtCompanyUserResponse.setJob(mtCompanyUser.getJob());
                //设置公司用户基础信息
                if (userResponse != null) {
                    mtCompanyUserResponse.setUserId(mtCompanyUser.getUserId());
                    mtCompanyUserResponse.setUserMobile(userResponse.getMobile());
                    mtCompanyUserResponse.setUserName(userResponse.getName());
                }
                //设置公司用户角色名称
                Role role = userIdToRole.get(mtCompanyUser.getUserId());
                if(role != null){
                    mtCompanyUserResponse.setRoleName(role.getName());
                }

                //设置用户操作证状态 0：无证，1：待审核，2：审核未通过，3：审核通过，4：超期
                LiftCertificate liftCertificate = userIdToLiftCertificate.get(mtCompanyUser.getUserId());
                if(liftCertificate != null){
                    mtCompanyUserResponse.setCertificateStatus(liftCertificate.getStatus());
                }
                mtCompanyUserResponseList.add(mtCompanyUserResponse);
            }
        }
        //分页数据更换
        mtCompanyUserPage.setRecords(mtCompanyUserResponseList);
        return RestResponse.ok(mtCompanyUserResponseList, ApiConstants.RESULT_SUCCESS, "获取公司成员列表成功");
    }

}
